#! /usr/bin/env python

class RemoteTemplate(object):
    def __init__(self, source):
        self.source = source
        from wcs.qommon.misc import simplify
        import os, urlparse
        self.cache_key = simplify(urlparse.urlunparse(urlparse.urlparse(self.source)[:3] + ('', '', '')))
        self.cache_dirpath = os.path.join(publisher.app_dir, 'skeleton-cache')
        self.cache_filepath = os.path.join(self.cache_dirpath, self.cache_key)
        self.theme_skeleton_url = publisher.get_site_option('theme_skeleton_url')
        self.css_variant = publisher.get_site_option('css_variant', 'variables')
        self.skip_cache = (request.response.status_code != 200)

    def get_template_content(self):
        import time, os, threading
        if self.skip_cache:
            return self.update_content()
        CACHE_REFRESH_TIMEOUT = 300
        try:
            cache_filepath_mtime = min((
                os.stat(self.cache_filepath).st_mtime,
                os.lstat(self.cache_filepath).st_mtime))
        except OSError: # missing file
            return self.update_content()
        else:
            template_content = open(self.cache_filepath).read()
            if time.time() > (cache_filepath_mtime + CACHE_REFRESH_TIMEOUT):
                threading.Thread(target=self.update_content).start()
            return template_content

    def update_content(self):
        import requests

        r = requests.get(self.theme_skeleton_url, params={'source': self.source, 'format': 'ezt'}, verify=False)
        r.raise_for_status()

        body = """
             [if-any gauche]
             <div id="gauche">
             [gauche]
             </div>
             [end]

             [if-any bigdiv]<div id="[bigdiv]" [if-any breadcrumb]class="has-breadcrumb"[end]>[end]

             [if-any title]<h2>[title]</h2>[end]
             [body]
             [if-any bigdiv]</div>[end]
        """

        extra_head = """
[script]
<script type="text/javascript" src="/themes/publik-base/static/dataview.js"></script>
"""

        extra_top_head = ''

        user_info = """
          [if-any user]
          <span class="logged-in">
            <span class="connected-user">[session_user_display_name]</span>
            <a accesskey="o" class="logout" href="[root_url]logout">D&eacute;connexion</a>
          </span>
          [else]
          <span class="login"><a accessey="2" href="[root_url]login/">Connexion</a> <span class="sep">/</span> <a href="[root_url]register/">Inscription</a></span>
          [end]
          """

        first_user_link = """
          [if-any user]<span class="connected-user">[session_user_display_name]</span>
          [else]
          <span class="login"><a accessey="2" href="[root_url]login/">Se connecter</a>
          [end]"""

        second_user_link = """
          [if-any user]<span class="logout"><a accesskey="o" class="logout" href="[root_url]logout">Se d&eacute;connecter</a></span>
          [else]<span class="register"><a href="[root_url]register/">Cr&eacute;er un compte</a></span>
          [end]"""

        template_content = r.text.encode('utf-8')
        template_content = template_content.replace('[if-any content][content][end]', body)
        template_content = template_content.replace('[if-any extra-top-head][extra-top-head][end]', extra_top_head)
        template_content = template_content.replace('[if-any extra-head][extra-head][end]', extra_head)
        template_content = template_content.replace('[if-any user-info][user-info][end]', user_info)
        template_content = template_content.replace(
                '[if-any first-user-link][first-user-link][end]', first_user_link)
        template_content = template_content.replace(
                '[if-any second-user-link][second-user-link][end]', second_user_link)
        if not self.skip_cache:
            self.cache(template_content, page_id=r.headers.get('X-Combo-Page-Id'))
        return template_content

    def cache(self, template_body, page_id=None):
        import os
        from qommon.storage import atomic_write
        if not os.path.exists(self.cache_dirpath):
            os.mkdir(self.cache_dirpath)
        if page_id:
            # if there's a page id we store the content under the page id
            # filename and create a symlink from the cache key
            cache_filepath = os.path.join(self.cache_dirpath, page_id)
            atomic_write(cache_filepath, template_body)
            # always recreate symlink to get an updated mtime
            if os.path.exists(self.cache_filepath):
                os.unlink(self.cache_filepath)
            os.symlink(page_id, self.cache_filepath)
        else:
            atomic_write(self.cache_filepath, template_body)

template_content = RemoteTemplate(request.get_url()).get_template_content()
